@@ -3,7 +3,7 @@
 # Herein, the symbol $self is used to refer to the object that's being passed around.
 
 package HTML::CalendarMonthSimple;
-$HTML::CalendarMonthSimple::VERSION = "1.21";
+$HTML::CalendarMonthSimple::VERSION = "1.23";
 use strict;
 use Date::Calc;
 
@@ -16,9 +16,17 @@ sub new {
    my $class = shift; $class = ref($class) || $class;
    my $self = {}; %$self = @_; # Load ourselves up from the args
 
-   # Set the month and year to either args or today
-   ($self->{'month'})  || ($self->{'month'}  = (Date::Calc::Today)[1]);
-   ($self->{'year'})   || ($self->{'year'}   = (Date::Calc::Today)[0]);
+   # figure out the current date (which may be specified as today_year, et al
+   # then figure out which year+month we're supposed to display
+   {
+      my($year,$month,$date) = Date::Calc::Today();
+      $self->{'today_year'}  = $self->{'today_year'} || $year;
+      $self->{'today_month'} = $self->{'today_month'} || $month;
+      $self->{'today_date'}  = $self->{'today_date'} || $date;
+      $self->{'month'}       = $self->{'month'} || $self->{'today_month'};
+      $self->{'year'}        = $self->{'year'}  || $self->{'today_year'};
+      $self->{'monthname'}   = Date::Calc::Month_to_Text($self->{'month'});
+   }
 
    # Some defaults
    $self->{'border'}             = 5;
@@ -38,9 +46,6 @@ sub new {
    $self->{'header'} = sprintf("<center><font size=+2>%s %d</font></center>",
                                Date::Calc::Month_to_Text($self->{'month'}),$self->{'year'});
 
-   # Get the monthname now so monthname() is fast and efficient
-   $self->{'monthname'} = Date::Calc::Month_to_Text($self->{'month'});
-
    # Initialize the (empty) cell content so the keys are representative of the month
    foreach my $datenumber ( 1 .. Date::Calc::Days_in_Month($self->{'year'},$self->{'month'}) ) {
       $self->{'content'}->{$datenumber}          = '';
@@ -56,16 +61,25 @@ sub new {
 }
 
 
-
 sub as_HTML {
    my $self = shift;
+   my %params = @_; 
    my $html = '';
    my(@days,$weeks,$WEEK,$DAY);
 
    # To make the grid even, pad the start of the series with 0s
    @days = (1 .. Date::Calc::Days_in_Month($self->year(),$self->month() ) );
-   foreach (1 .. (Date::Calc::Day_of_Week($self->year(),$self->month(),1)%7) ) {
-      unshift(@days,0);
+   if ($self->weekstartsonmonday()) {
+       foreach (1 .. (Date::Calc::Day_of_Week($self->year(),
+                                              $self->month(),1) -1 )) {
+          unshift(@days,0);
+       }
+   }
+   else {
+       foreach (1 .. (Date::Calc::Day_of_Week($self->year(),
+                                              $self->month(),1)%7) ) {
+          unshift(@days,0);
+       }
    }
    $weeks = int((scalar(@days)+6)/7);
    # And pad the end as well, to avoid "uninitialized value" warnings
@@ -110,8 +124,9 @@ sub as_HTML {
    my $todaycellclass = $self->todaycellclass() || $self->cellclass();
    my $headerclass = $self->headerclass() || $self->cellclass();
    my $nowrap = $self->nowrap();
+
    # Get today's date, in case there's a todaycolor()
-   my($todayyear,$todaymonth,$todaydate) = Date::Calc::Today();
+   my($todayyear,$todaymonth,$todaydate) = ($self->today_year(),$self->today_month(),$self->today_date());
 
    # the table declaration - sharpborders wraps the table inside a table cell
    if ($sharpborders) {
@@ -151,39 +166,88 @@ sub as_HTML {
    if ($self->showweekdayheaders) {
       my $celltype = $self->weekdayheadersbig() ? "th" : "td";
       my @weekdays = $self->weekdays();
-      $html .= "<tr>\n";
-      $html .= "<$celltype";
-      $html .= " bgcolor=\"$weekendheadercolor\"" if defined $weekendheadercolor;
-      $html .= " class=\"$weekendcellclass\"" if defined $weekendcellclass;
-      $html .= ">";
-      $html .= "<font color=\"$weekendheadercontentcolor\">" if defined $weekendheadercontentcolor;
-      $html .= $self->sunday();
-      $html .= "</font>" if defined $weekendheadercontentcolor;
-      $html .= "</$celltype>\n";
+
+      my $saturday_html = "<$celltype"
+                        . ( defined $weekendheadercolor 
+                            ? qq| bgcolor="$weekendheadercolor"| 
+                            : '' )
+                        . ( defined $weekendcellclass 
+                            ? qq| class="$weekendcellclass"| 
+                            : '' ) 
+                        . ">"
+                        . ( defined $weekendheadercontentcolor 
+                            ? qq|<font color="$weekendheadercontentcolor">| 
+                            : '' ) 
+                        . $self->saturday()
+                        . ( defined $weekendheadercontentcolor 
+                            ? qq|</font>|
+                            : '' )
+                        . "</$celltype>\n";
+
+      my $sunday_html   = "<$celltype"
+                        . ( defined $weekendheadercolor 
+                            ? qq| bgcolor="$weekendheadercolor"| 
+                            : '' )
+                        . ( defined $weekendcellclass 
+                            ? qq| class="$weekendcellclass"| 
+                            : '' ) 
+                        . ">"
+                        . ( defined $weekendheadercontentcolor 
+                            ? qq|<font color="$weekendheadercontentcolor">| 
+                            : '' ) 
+                        . $self->sunday()
+                        . ( defined $weekendheadercontentcolor 
+                            ? qq|</font>|
+                            : '' )
+                        . "</$celltype>\n";
+      
+      my $weekday_html = '';
       foreach (@weekdays) { # draw the weekday headers
-         $html .= "<$celltype";
-         $html .= " bgcolor=\"$weekdayheadercolor\"" if defined $weekdayheadercolor;
-         $html .= " class=\"$weekdaycellclass\"" if defined $weekdaycellclass;
-         $html .= ">";
-         $html .= "<font color=\"$weekdayheadercontentcolor\">" if defined $weekdayheadercontentcolor;
-         $html .= "$_";
-         $html .= "</font>" if defined $weekdayheadercontentcolor;
-         $html .= "</$celltype>\n";
+
+         $weekday_html  .= "<$celltype"
+                        . ( defined $weekendheadercolor 
+                            ? qq| bgcolor="$weekdayheadercolor"| 
+                            : '' )
+                        . ( defined $weekendcellclass 
+                            ? qq| class="$weekdaycellclass"| 
+                            : '' ) 
+                        . ">"
+                        . ( defined $weekdayheadercontentcolor 
+                            ? qq|<font color="$weekdayheadercontentcolor">| 
+                            : '' ) 
+                        . $_
+                        . ( defined $weekdayheadercontentcolor 
+                            ? qq|</font>|
+                            : '' )
+                        . "</$celltype>\n";
+      }
+
+      $html .= "<tr>\n";
+      if ($self->weekstartsonmonday()) {
+        $html .= $weekday_html
+              .  $saturday_html
+              .  $sunday_html;
+      }
+      else {
+        $html .= $sunday_html
+              .  $weekday_html
+              .  $saturday_html;
       }
-      $html .= "<$celltype";
-      $html .= " bgcolor=\"$weekendheadercolor\"" if defined $weekendheadercolor;
-      $html .= " class=\"$weekendcellclass\"" if defined $weekendcellclass;
-      $html .= ">";
-      $html .= "<font color=\"$weekendheadercontentcolor\">" if defined $weekendheadercontentcolor;
-      $html .= $self->saturday();
-      $html .= "</font>" if defined $weekendheadercontentcolor;
-      $html .= "</$celltype>\n";
       $html .= "</tr>\n";
    }
+
+   my $_saturday_index = 6;
+   my $_sunday_index   = 0;
+   if ($self->weekstartsonmonday()) {
+       $_saturday_index = 5;
+       $_sunday_index   = 6;
+   }
    # now do each day, the actual date-content-containing cells
    foreach $WEEK (0 .. ($weeks-1)) {
       $html .= "<TR>\n";
-      foreach $DAY (0 .. 6) {
+
+      
+      foreach $DAY ( 0 .. 6 ) {
          my($thiscontent,$thisday,$thisbgcolor,$thisbordercolor,$thiscontentcolor,$thiscellclass);
          $thisday = $days[((7*$WEEK)+$DAY)];
 
@@ -211,7 +275,7 @@ sub as_HTML {
                                                 $thiscontentcolor = $self->datecontentcolor($thisday) || $todaycontentcolor;
                                                 $thiscellclass = $self->datecellclass($thisday) || $todaycellclass;
                                               }
-         elsif (($DAY == 0) || ($DAY == 6))   { $thisbgcolor = $self->datecolor($thisday) || $weekendcolor;
+         elsif (($DAY == $_sunday_index) || ($DAY == $_saturday_index))   { $thisbgcolor = $self->datecolor($thisday) || $weekendcolor;
                                                 $thisbordercolor = $self->datebordercolor($thisday) || $weekendbordercolor;
                                                 $thiscontentcolor = $self->datecontentcolor($thisday) || $weekendcontentcolor;
                                                 $thiscellclass = $self->datecellclass($thisday) || $weekendcellclass;
@@ -257,6 +321,7 @@ sub as_HTML {
 }
 
 
+
 sub sunday {
    my $self = shift;
    my $newvalue = shift;
@@ -579,6 +644,21 @@ sub monthname {
    return $self->{'monthname'};
 }
 
+sub today_year {
+   my $self = shift;
+   return $self->{'today_year'};
+}
+
+sub today_month {
+   my $self = shift;
+   return $self->{'today_month'};
+}
+
+sub today_date {
+   my $self = shift;
+   return $self->{'today_date'};
+}
+
 
 sub header {
    my $self = shift;
@@ -658,6 +738,14 @@ sub headerclass {
     return $self->{'headerclass'};
 }
 
+sub weekstartsonmonday {
+    my $self = shift;
+    my $newvalue = shift;
+    if (defined($newvalue)) { $self->{'weekstartsonmonday'} = $newvalue; }
+    return $self->{'weekstartsonmonday'} ? 1 : 0;
+}
+
+
 ### the following methods are internal-use-only methods
 
 # _date_string_to_numeric() takes a date string (e.g. 5, 'wednesdays', or '3friday')
@@ -668,14 +756,15 @@ sub _date_string_to_numeric {
    my $self = shift;
    my $date = shift || return ();
 
+   my($which,$weekday);
    if ($date =~ m/^\d\.*\d*$/) { # first and easiest, simple numerics
       return int($date);
    }
-   elsif (my($which,$weekday) = ($date =~ m/^(\d)([a-zA-Z]+)$/)) {
+   elsif (($which,$weekday) = ($date =~ m/^(\d)([a-zA-Z]+)$/)) {
       my($y,$m,$d) = Date::Calc::Nth_Weekday_of_Month_Year($self->year(),$self->month(),Date::Calc::Decode_Day_of_Week($weekday),$which);
       return $d;
    }
-   elsif (my($weekday) = ($date =~ m/^(\w+)s$/i)) {
+   elsif (($weekday) = ($date =~ m/^(\w+)s$/i)) {
       $weekday = Date::Calc::Decode_Day_of_Week($weekday); # now it's the numeric weekday
       my @dates;
       foreach my $which (1..5) {
@@ -724,24 +813,36 @@ This module requires the Date::Calc module, which is available from CPAN if you
 
 =head1 new(ARGUMENTS)
 
-Naturally, new() returns a newly constructed calendar object. Recognized arguments are 'year' and 'month', to specify which month's calendar will be used. If either is omitted, the current value is used. An important note is that the month and the year are NOT the standard C or Perl -- use a month in the range 1-12 and a real year, e.g. 2001.
+Naturally, new() returns a newly constructed calendar object.
+
+The optional constructor arguments 'year' and 'month' can specify which month's calendar will be used. If either is omitted, the current value (e.g. "today") is used. An important note is that the month and the year are NOT the standard C or Perl -- use a month in the range 1-12 and a real year, e.g. 2001.
+
+The arguments 'today_year', 'today_month', and 'today_date' may also be specified, to specify what "today" is. If not specified, the system clock will be used. This is particularly useful when the todaycolor() et al methods are used, and/or if you're dealing with multiple timezones. Note that these arguments change what "today" is, which means that if you specify a today_year and a today_month then you are effectively specifying a 'year' and 'month' argument as well, though you can also specify a year and month argument and override the "today" behavior.
 
    # Examples:
    # Create a calendar for this month.
    $cal = new HTML::CalendarMonthSimple();
-   # One for a specific month/year
+   # A calendar for a specific month/year
    $cal = new HTML::CalendarMonthSimple('month'=>2,'year'=>2000);
-   # One for "the current month" in 1997
-   $cal = new HTML::CalendarMonthSimple('year'=>1997);
+   # Pretend that today is June 10, 2000 and display the "current" calendar
+   $cal = new HTML::CalendarMonthSimple('today_year'=>2000,'today_month'=>6,'today_date'=>10);
 
 
 =head1 year()
 
 =head1 month()
 
+=head1 today_year()
+
+=head1 today_month()
+
+=head1 today_date()
+
 =head1 monthname()
 
-These methods simply return the year/month of the calendar. monthname() returns the text name of the month, e.g. "December".
+These methods simply return the year/month/date of the calendar, as specified in the constructor.
+
+monthname() returns the text name of the month, e.g. "December".
 
 
 
@@ -814,6 +915,18 @@ It's okay to continue modifying the calendar after calling as_HTML(). My guess i
 
 
 
+=head1 weekstartsonmonday([1|0])
+
+By default, calendars are displayed with Sunday as the first day of the week (American style). Most of the world prefers for calendars to start the week on Monday. This method selects which type is used: 1 specifies that the week starts on Monday, 0 specifies that the week starts on Sunday (the default). If no value is given at all, the current value (1 or 0) is returned.
+
+   # Example:
+   $cal->weekstartsonmonday(1); # switch over to weeks starting on Monday
+   $cal->weekstartsonmonday(0); # switch back to the default, where weeks start on Sunday
+
+   # Example:
+   print "The week starts on " . ($cal->weekstartsonmonday() ? 'Sunday' : 'Monday') . "\n";
+
+
 =head1 setdatehref(DATE,URL_STRING)
 
 =head1 getdatehref(DATE)
@@ -1112,6 +1225,10 @@ Ver 1.20 was a mistake on my part and was immediately superseded by 1.21.
 
 Changes in 1.21: Fixed the internals of setcontent() et al (see the method's doc for details). Made getdatehref(), setdatehref(), and datecolor() et al, able to handle weekdays in addition to numeric dates.
 
+Changes in 1.22: Added the much-desired weekstartsonmonday() method. Now weeks can start on Monday and end with the weekend, instead of the American style of starting on Sunday.
+
+Changes in 1.23: Added today_year() et al. "Today" can now be overridden in the constructor.
+
 
 
 =head1 AUTHORS, CREDITS, COPYRIGHTS
@@ -1146,3 +1263,7 @@ Blair Zajac <blair@orcaware.com> provided the fixes for 1.19
 
 Thanks to Kurt <kurt@otown.com> for the bug report that made all the new stuff in 1.21 possible.
 
+Many thanks to Stefano Rodighiero <larsen@libero.it> for the code that made weekstartsonmonday() possible. This was a much-requested feature that will make many people happy!
+
+Dan Boitnott <dboitnot@yahoo.com> provided today_year() et al in 1.23
+
